@@ -1,10 +1,10 @@
 package Lingua::Han::PinYin;
 
 use strict;
-use vars qw($VERSION);
-$VERSION = '0.13';
+use warnings;
+our $VERSION = '0.14';
 
-use File::Spec;
+use File::Spec ();
 use Lingua::Han::Utils qw/Unihan_value/;
 
 sub new {
@@ -17,8 +17,14 @@ sub new {
     my $file = File::Spec->catfile( $dir, 'Mandarin.dat' );
     open( FH, $file ) or die "$file: $!";
 
-    while (<FH>) {
-        my ( $uni, $py ) = split(/\s+/);
+    while (my $line = <FH>) {
+        chomp($line);
+        my ( $uni, $py );
+        if ( $self->{duoyinzi} ) {
+            ( $uni, $py ) = split(/\s+/, $line, 2);
+        } else {
+            ( $uni, $py ) = split(/\s+/, $line);
+        }
         $py{$uni} = $py;
     }
     close(FH);
@@ -31,11 +37,10 @@ sub han2pinyin1 {
     my $code = Unihan_value($word);
     my $value = $self->{'py'}->{$code};
     if (defined $value) {
-       $value =~ s/\d//isg unless ($self->{'tone'});
-       $value = lc $value;
+        $value = $self->_fix_val( $value );
     } else {
-       # not found in dictionary, return original word
-       $value = $word;
+        # not found in dictionary, return original word
+        $value = $word;
     }
     return $value;
 }
@@ -49,13 +54,13 @@ sub han2pinyin {
     foreach my $code (@code) {
         my $value = $self->{'py'}->{$code};
         if ( defined $value ) {
-            $value =~ s/\d//isg unless ( $self->{'tone'} );
+            $value = $self->_fix_val( $value );
         }
         else {
             # if it's not a Chinese, return original word
             $value = pack( "U*", hex $code );
         }
-        push @result, lc $value;
+        push @result, $value;
     }
 
     return wantarray ? @result : join( '', @result );
@@ -72,6 +77,22 @@ sub gb2pinyin {
     return $hanzi;
 }
 
+sub _fix_val {
+    my ( $self, $value ) = @_;
+    
+    unless ($self->{'tone'}) {
+        $value =~ s/\d//isg;
+        if ( $self->{duoyinzi} ) { # remove duplication
+            my @duoyinzi = split(/\s+/, $value);
+            my %saw;
+            my @out = grep(!$saw{$_}++, @duoyinzi);
+            $value = join(' ', @out);
+        }
+    }
+    
+    return lc($value);
+}
+
 1;
 __END__
 
@@ -86,10 +107,13 @@ Lingua::Han::PinYin - Retrieve the Mandarin(PinYin) of Chinese character(HanZi).
   use Lingua::Han::PinYin;
   
   my $h2p = new Lingua::Han::PinYin();
+  
+  # han2pinyin
   print $h2p->han2pinyin(""); # wo
+  my @result = $h2p->han2pinyin(""); # @result = ('ai', 'ni');
+  
   # if you are sure to pass 1 Chinese letter at a time, han2pinyin1 is faster
   print $h2p->han2pinyin1(""); # wo
-  my @result = $h2p->han2pinyin(""); # @result = ('ai', 'ni');
   # if you are sure your encoding is GB2312, gb2pinyin is faster
   print $h2p->gb2pinyin("I love ƴ Ah"); # I love hanyupinyin Ah
 
@@ -99,15 +123,15 @@ Lingua::Han::PinYin - Retrieve the Mandarin(PinYin) of Chinese character(HanZi).
   my @result = $h2p->han2pinyin(""); # @result = ('ai4', 'ni3');
   print $h2p->han2pinyin("ֵ"); #lin2dao4
   print $h2p->han2pinyin("I love  a"); #i love yuruihua a
+  
+  # for polyphone(duoyinzi)
+  my $h2p = new Lingua::Han::PinYin(duoyinzi => 1, tone => 1);
+  print $h2p->han2pinyin(""); # 'xing2 hang2 xing4 hang4 heng2'
 
 =head1 DESCRIPTION
 
 There is a Chinese document @ L<http://www.fayland.org/project/Han-PinYin/>. It tells why and how I write this module.
 
-=head1 RESTRICTIONS
-
-if the character is polyphone(DuoYinZi), we can B<NOT> point out the correct one.
-
 =head1 RETURN VALUE
 
 Usually, it returns its pinyin/spell. It includes more than 20,000 words (from Unicode.org Unihan.txt, version 4.1.0).
@@ -122,6 +146,10 @@ if not(I mean it's not a Chinese character), returns the original word;
 
 default is 0. if tone is needed, plz set this to 1.
 
+=item duoyinzi => 1|0
+
+default is 0.
+
 =back 
 
 =head1 CAVEAT
@@ -132,15 +160,15 @@ The ascii 'v' is used instead of the unicode 'yu' Since version 0.06.
 
 L<Unicode::Unihan>
 
-=head1 AUTHOR
+=head1 AUTHORS
 
-Fayland, fayland@gmail.com
+Fayland Lam, C<< <fayland at gmail.com> >>
 
-feel free to contact me.
+Tong Sun, C<< <suntong at cpan.org> >>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2005 Fayland All rights reserved.
+Copyright (c) 2005-2009 *AUTHORS* All rights reserved.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
